package main

import (
	"encoding/json"
	"io/ioutil"
	"net/http"
	"net/http/httptest"
	"os"
	"os/exec"
	"testing"

	corev2 "github.com/sensu/sensu-go/api/core/v2"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestFormattedEventAction(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")

	action := formattedEventAction(event)
	assert.Equal("RESOLVED", action)

	event.Check.Status = 1
	action = formattedEventAction(event)
	assert.Equal("ALERT", action)
}

func TestChomp(t *testing.T) {
	assert := assert.New(t)

	trimNewline := chomp("hello\n")
	assert.Equal("hello", trimNewline)

	trimCarriageReturn := chomp("hello\r")
	assert.Equal("hello", trimCarriageReturn)

	trimBoth := chomp("hello\r\n")
	assert.Equal("hello", trimBoth)

	trimLots := chomp("hello\r\n\r\n\r\n")
	assert.Equal("hello", trimLots)
}

func TestEventKey(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")
	eventKey := eventKey(event)
	assert.Equal("entity1/check1", eventKey)
}

func TestEventSummary(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")
	event.Check.Output = "disk is full"

	eventKey := eventSummary(event, 100)
	assert.Equal("entity1/check1:disk is full", eventKey)

	eventKey = eventSummary(event, 5)
	assert.Equal("entity1/check1:disk ...", eventKey)
}

func TestFormattedMessage(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")
	event.Check.Output = "disk is full"
	event.Check.Status = 1
	formattedMsg := formattedMessage(event)
	assert.Equal("ALERT - entity1/check1:disk is full", formattedMsg)
}

func TestMessageColor(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")

	event.Check.Status = 0
	color := messageColor(event)
	assert.Equal("00FF00", color)

	event.Check.Status = 1
	color = messageColor(event)
	assert.Equal("FFFF00", color)

	event.Check.Status = 2
	color = messageColor(event)
	assert.Equal("FF0000", color)
}

func TestMessageStatus(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")

	event.Check.Status = 0
	status := messageStatus(event)
	assert.Equal("Resolved", status)

	event.Check.Status = 1
	status = messageStatus(event)
	assert.Equal("Warning", status)

	event.Check.Status = 2
	status = messageStatus(event)
	assert.Equal("Critical", status)
}

func TestSendMessage(t *testing.T) {
	assert := assert.New(t)
	event := corev2.FixtureEvent("entity1", "check1")

	var apiStub = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		body, _ := ioutil.ReadAll(r.Body)
		expectedBody := `{"@type":"MessageCard","@context":"https://schema.org/extensions","summary":"Sensu alert card","title":"Sensu - RESOLVED","themeColor":"00FF00","sections":[{"text":"Test","activityTitle":"check1","activitySubtitle":"2021-11-17 02:00","facts":[{"name":"Sender:","value":"Sensu"},{"name":"Status:","value":"Resolved"},{"name":"Entity:","value":"entity1"}]}],"potentialAction":[{"@type":"OpenUri","name":"View in Sensu","targets":[{"os":"default","uri":"http://localhost:3000"}]}]}`
		assert.Equal(expectedBody, string(body))
		w.WriteHeader(http.StatusOK)
		_, err := w.Write([]byte(`1`))
		require.NoError(t, err)
	}))

	config.teamsSender = "Sensu"
	config.teamsSensuURL = "http://localhost:3000"
	config.teamswebHookURL = apiStub.URL
	config.teamsIsTest = "true"
	config.teamsDescriptionTemplate = "{{ .Check.Output }}"
	err := sendMessage(event)
	assert.NoError(err)
}

func TestMain(t *testing.T) {
	if os.Getenv("BE_TEST") == "1" {
		assert := assert.New(t)
		file, _ := ioutil.TempFile(os.TempDir(), "sensu-handler-teams-")
		defer func() {
			_ = os.Remove(file.Name())
		}()

		event := corev2.FixtureEvent("entity1", "check1")
		eventJSON, _ := json.Marshal(event)
		_, err := file.WriteString(string(eventJSON))
		require.NoError(t, err)
		require.NoError(t, file.Sync())
		_, err = file.Seek(0, 0)
		require.NoError(t, err)
		os.Stdin = file
		requestReceived := false

		var apiStub = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
			requestReceived = true
			w.WriteHeader(http.StatusOK)
			_, err := w.Write([]byte(`1`))
			require.NoError(t, err)
		}))

		oldArgs := os.Args
		os.Args = []string{"teams", "-w", apiStub.URL}
		defer func() { os.Args = oldArgs }()

		main()
		assert.True(requestReceived)
		return
	}

	assert2 := assert.New(t)
	cmd := exec.Command(os.Args[0], "-test.run=TestMain")
	cmd.Env = append(os.Environ(), "BE_TEST=1")
	err2 := cmd.Run()
	if e, ok := err2.(*exec.ExitError); ok && !e.Success() {
		assert2.True(false)
		return
	}
	assert2.True(true)
}
